#!/usr/bin/python3
# -*- coding: utf-8 -*-
'''
________________________________________________________________________
 a_tool.py                                                        
 This file is free software under         
     the terms of the GPL without any warranty - see the file COPYING 
 Diese Datei ist Freie Software gemäß
             der GPL ohne jegliche Garantie - siehe die Datei COPYING 
________________________________________________________________________

 a_tool.py stellt Tools für die anderen absenzen-Module zur Verfügung
  Ziel: Alle session-bezogenen dorthin auslagern
        Alle db-bezogenen in eigene a_db.py auslagern
________________________________________________________________________
'''
##Importe
import sys, os, random, hashlib
import codecs
import sqlite3
import time, datetime
import cgi, cgitb
cgitb.enable()

from a_konstanten import *
import a_db

def lesbare_betroffene(betroffene,klassehervorheben=False):
    b_liste=betroffene.split()
    for i in range(len(b_liste)):
        if b_liste[i][0]=="@" and len(b_liste[i])>1:
            if klassehervorheben:
                b_liste[i]='<i>'+b_liste[i][1:]+'</i>'
            else:
                b_liste[i]=b_liste[i][1:]
    betroffene=','.join( b_liste)
    return betroffene

def entferne_titel(text):
    for boese in removeit: # aus a_konstanten.py
        text=text.replace(boese+".","")
        text=text.replace(boese,"")
    return text

def lf2br(kommentar,zeilenlimit=0): # zeilenlimit<0: letzte n Zeilen, >0: erste n Zeilen, =0: alle
    if not zeilenlimit:
        return  '<br>'.join(kommentar.split("\n"))
    if zeilenlimit<0:
        return  '<br>'.join(  kommentar.split("\n")[zeilenlimit:]     )
    return      '<br>'.join(  kommentar.split("\n")[:zeilenlimit]     )
    
def istisodatum(datum):
    if  len(datum)==10 and \
        datum[:2]=='20' and  \
        datum[4]=='-' and  \
        datum[7]=='-' and  \
        datum[2:4].isdigit() and \
        datum[5:7].isdigit() and \
        datum[8:10].isdigit() and \
        int(datum[5:7])<13 and \
        int(datum[8:10])<32:
            return True
    return False

def isodatum2datum(isodatum):
    if len(isodatum)!=10:     return '?nodate?'
    t,m,j= isodatum[8:],isodatum[5:7],isodatum[:4]
    return t+"."+m+"."+j

def isodatum2tagmonat(isodatum):
    if len(isodatum)!=10:     return '?nodate?'
    t,m= isodatum[8:],isodatum[5:7]
    return t+"."+m+"."

def datum2isodatum(datum):
    if len(datum)!=10:    return '?nodate?'
    t,m,j= datum[:2],datum[3:5],datum[6:]
    return j+"-"+m+"-"+t

def isodatum2int(isodatum):
    t,m,j= isodatum[8:],isodatum[5:7],isodatum[:4]
    return int(t),int(m),int(j)

def wochentag(isodatum):
    ts=time.strptime(isodatum,'%Y-%m-%d')
    return ts.tm_wday
    
def isodatum_plus_tage(isodatum,tage):
    t,m,y=isodatum2int(isodatum)
    delta =datetime.timedelta(tage)
    mydate=datetime.date(y,m,t)+delta
    return mydate.isoformat()
    
def kalenderwoche(isodatum):
    t,m,y=isodatum2int(isodatum)
    mydate=datetime.date(y,m,t)
    return mydate.isocalendar()[1]
    
def kalendermonat(isodatum):
    t,m,y=isodatum2int(isodatum)
    mydate=datetime.date(y,m,t)
    return mydate.month
    
def ist_freier_tag(datum):
    ob,warum=ist_freier_tag_warum(datum)
    return ob
    
def ist_freier_tag_warum(datum):
    rowdict=a_db.get_one_rowdict('''
            SELECT count(name),name
              FROM noschool 
             WHERE (? BETWEEN erstertag AND letztertag)
                OR (? BETWEEN erstertag AND letztertag)''',(datum,'9999'+datum[4:]))
    if rowdict:
        if rowdict[0]>0:
            return True, rowdict[1]
    return False,''

def letzter_halbjahresbeginn():
    startmonat=9 if timestamp.tm_mon>8 or timestamp.tm_mon<3 else 1
    starttag=10 if startmonat==9 else 5
    if timestamp.tm_mon<9 and startmonat>8: # wenn jetzt 1. HJ, Anzeige soll 2. HJ zeigen: also früher im Schuljahr = letztes Kalenderjahr
        startjahr=timestamp.tm_year-1
    elif timestamp.tm_mon>8 and startmonat<9: # und umgekehrt
        startjahr=timestamp.tm_year+1
    else:
        startjahr=timestamp.tm_year
    return startjahr,startmonat,starttag

    

def reales_schuljahr():
    t,m,j=isodatum2int(heutedatum)
    return j if m>=9 else j-1
    
def sortierbar(name):
    sname=name.lower()
    for such, ersetz in umlautersetzungstabelle:
        sname=sname.replace(such,ersetz)
    return sname

def alpha_einlesen():
    try:
        rowdict_list=a_db.get_rowdict_list('SELECT buchstaben FROM alphabetisch ORDER BY buchstaben')
        a_konstanten.alphabetisch=['Aktuelle']
        for row in rowdict_list:
            a_konstanten.alphabetisch.append(row["buchstaben"])
    except:
        pass

def zufallscode():
    '''Es wird eine Zufallszahl erzeugt und 
    mit einem Prüfcode im Stil der ISBN_Nummer (aber nicht identisch)
    versehen. Es werden dadurch Einzelziffernfehler und Zifferndreher sicher erkannt.
    Außerdem werden Zahlen mit drei aufeinanderfolgenden gleichen Ziffern ausgeschlossen.
    Gesamtlänge: 7 Stellen
    Aufbau: 123456X'''
    triple=True
    while triple:
        zahl=str(random.randint(100000,999999))
        triple=False
        for i in range(4):
            if zahl[i]==zahl[i+1] and zahl[i+1]==zahl[i+2]:
                triple=True
    summe=0
    faktor=[1,3,7,1,3,7]
    buchstabe='ACEFGHKLMPRSTW'
    for i in range(len(zahl)):
        summe+=int(zahl[i])*faktor[i]
    return zahl+buchstabe[summe%13]
    
def schueler_eintragen(name,kbez,js,klasse="",do_commit=True):
    logtext=""
    logerror=""
    stummel=False
    ort=name.find("(")
    if ort>0:
        logtext+='Klammer gefunden! Modifiziere "'+name+'" zu '
        name=name[:ort].strip() # nimm nur die Zeichen vor der Klammer und entferne führenden und endenden Whitespace
        stummel=True
        logtext+='"'+name+'"'
    sortiername=sortierbar(name)
    pid,pjs,pkl=db_search_person(name,js)                   # gibt es den Schülernamen schon?
    if pid<1 and stummel:
        pid,pjs,pkl=db_search_person(name,js,like=True)     # zweiter Versuch um abgeschnittene Namen zu umgehen
    if pid<1:
        logtext+='Neuer Schüler! '
        codeexistiert=True
        while codeexistiert:
            zugriffscode=zufallscode()
            codeexistiert=a_db.get_existenz('''SELECT pid FROM personen WHERE zugriffscode=?''',(zugriffscode,))
        apdatum='9999-99-99'
        if not klasse and js>10:
            klasse="Q"+str(js)
        a_db.execute('''INSERT INTO personen (name,js,klasse,apdatum,zugriffscode,sortiername) VALUES (?,?,?,?,?,?)''',(name,js,klasse,apdatum,zugriffscode,sortiername),do_commit)
        pid,pjs,pkl=db_search_person(name,js)
    elif pjs!=js or pkl=="":            
        if js>0:
            if not klasse and js>10:
                klasse="Q"+str(js)
            a_db.execute('''UPDATE personen SET js=?, klasse=? WHERE pid=?''',(js,klasse,pid),do_commit)
            if pjs!=js:
                logerror+="Jahrgangswechsel bei Schüler "+name+" von "+str(pjs)+" in "+str(js)+" wegen Kurs:"+kbez+' bzw. Klasse:'+klasse
        elif klasse!="":
            a_db.execute('''UPDATE personen SET klasse=? WHERE pid=?''',(klasse,pid),do_commit)
            logtext+='Schüler aktualisiert '
        else:
            logtext+='Schüler bekannt. Daten: js:'+str(pjs)+' kl.:'+pkl+' ' 
    else:
        logtext+='Schüler bekannt. Daten: js:'+str(pjs)+' kl.:'+pkl+' ' 

    # in der Q-Stufe auch die Schüler-Kurs-Beziehung eintragen:
    if len(kbez)>1:
        logtext+='Verknüpfe '+name+' '+str(pid)+' mit '+kbez
        try:
            a_db.execute('''INSERT INTO p2k VALUES (?,?)''',(pid,kbez),do_commit)
        except:
            logerror+='Fehler beim Einfügen der Beziehung  '+name+' mit '+kbez+'! Existiert Schüler zweimal in der Kursliste?'
    return logtext , logerror
    
def db_search_person(sname,wunschjs,like=False):
    '''gibt die ID und jahrgangsstufe der Person mit diesem Namen zurück
       Kommt der Name mehrfach vor, so wird der mit der vorgegebenen js bevorzugt!'''
    global db_cursor
    pid,js,klasse=-1 , 0 , ""
    if like:
        sname+="%"
    rowdict_list=a_db.get_rowdict_list('SELECT pid, js,klasse FROM personen  WHERE name LIKE ?',(sname,))
    for row in rowdict_list:
        temppid,tempjs,tempklasse = row
        if pid<0 or tempjs==wunschjs:   # Wenn dies der erste Treffer ist oder ein zu bevorzugender,
            pid,js,klasse=temppid,tempjs,tempklasse                 # dann die temporären Werte übernehmen
    return pid,js,klasse
    
def getcgiparameter(name,default=''):
    return __cgi__.getfirst(name,default)

def getcgiinteger(name,default=-1):
    temp=getcgiparameter(name)
    if temp.isdigit():
        return int(temp)
    else:
        return default

def getcgilist(name):
    return __cgi__.getlist(name)
    
def getcgiobject(parameter):
    return __cgi__[parameter]

def get_client_ip():
    try:
        ip=cgi.escape(os.environ["REMOTE_ADDR"])
    except:
        ip='127.0.0.1'
    return ip
        
def prt(data):
    data8=data.encode('utf-8')
    try:        sys.stdout.buffer.write(data8)
    except: sys.stdout.write(data8)
    
    
def lege_user_an(klarname,do_commit=True):
    user_nr=get_user_nr(klarname) # gibt's schon (prüft auch Aliase)? dann mit der bestehenden Nummer zurück
    if user_nr>0: return user_nr
    kuerzel=""
    while kuerzel=="":
        kuerzel=zufallscode()
        if a_db.get_existenz('SELECT kuerzel FROM users WHERE kuerzel=?',(kuerzel,)):
            kuerzel=""
    sname=sortierbar(klarname)
    pwd=kuerzel # initiales Passwort!
    hash=pwd2hash(kuerzel,pwd)
    rechte='0000' 
    a_db.execute('''
        INSERT INTO users (kuerzel,hash,rechte,klarname,sortiername) VALUES (?,?,?,?,?) ''' , (kuerzel, hash, rechte, klarname,sname),do_commit=do_commit)
    return get_user_nr(klarname)

def pwd2hash(kuerzel,pwd):      # rechnet einen Passworthash aus.
    pwws=kuerzel+"aB5EnT"+pwd   # mit (kuerzel+konstante als) Salt gegen (universelle) Rainbowtables
    return hashlib.md5(pwws.encode('utf-8')).hexdigest() 

def setze_user_passwort(klarname,kuerzel,pwd):
    kuerzel=kuerzel.lower()
    hash=pwd2hash(kuerzel,pwd)
    try:
        a_db.execute('UPDATE users SET kuerzel=?, hash=? WHERE klarname=?',(kuerzel,hash,klarname))
        return '+Passwort geändert für '+klarname
    except:
        return '-Fehler bei der Passwortänderung von '+klarname
    
def setze_user_gruppe(klarname,rechte):
    a_db.execute('UPDATE users SET rechte=? WHERE klarname=?',(rechte,klarname))
    db_conn.commit()

def setze_trechte(klarname,trechte):
    a_db.execute('UPDATE users SET trechte=? WHERE klarname=?',(trechte,klarname))
    db_conn.commit()

def bestimme_anfangsbuchstabenverteilung():
    debugtxt=''
    maximum=30
    # triviales Array aller Anfangsbuchstaben
    abst=['A','B','C','D','E','F','G','H','I','J','K','L','M','N',
          'O','P','Q','R','S','T','U','V','W','X','Y','Z','ZZZZZ']
    # Array, das für jeden Anfangsbuchstaben ein Tripel enthalten wird, 
    #  das für Unter/Mittel/Oberstufe die Anzahl der Schüler mit diesem Anfangsbuchstaben angibt
    abanzahl=[]
    for i in range(len(abst)-1):
        stufenarray=[]
        for intervall in [(5,7),(8,10),(11,13)]:
            von,bis=intervall
            rowdict_list=a_db.get_rowdict_list('SELECT count(name) FROM personen WHERE name>? AND name<? AND js>=? AND js<=?',(abst[i],abst[i+1],von,bis))
            for row in rowdict_list:
                stufenarray.append(int(row[0]))
        abanzahl.append(stufenarray)
        debugtxt+=abst[i]+' ' +str(stufenarray)+'\n'
    abanzahl.append([999,999,999])
    # aktrange gibt die aktuell akzeptierten Anfangsbuchstaben an
    aktrange=""
    # aktsum zählt für die Stufen, wie viele Schüler in diesem range vorkommen
    aktsum=[0,0,0]
    # Das Array alpha enthält schließlich die beschlossenen ranges 
    alpha=[]
    for i in range(len(abst)-1):
        aktrange+=abst[i]
        range_abschliessen=False
        for stufe in range(3):
            aktsum[stufe]+=abanzahl[i][stufe]
            if aktsum[stufe]+abanzahl[i+1][stufe]>maximum or (len(aktrange)>3 and i<23):
                range_abschliessen=True
        debugtxt+=abst[i]+' ' +str(abanzahl[i])+' ' +str(aktsum)+' '+aktrange+' Range voll:'+str(range_abschliessen)+'\n'
        if range_abschliessen:
            alpha.append(aktrange)
            aktrange=""
            aktsum=[0,0,0]
            
    a_db.execute('''DROP TABLE IF EXISTS alphabetisch''')
    a_db.execute('''CREATE TABLE alphabetisch (
                            buchstaben TEXT) ''')
    for i in range(len(alpha)):
       a_db.execute('INSERT INTO alphabetisch VALUES (?)',(alpha[i],))
    db_conn.commit()
    return debugtxt

def lese_htmlvorlage_ein(vorlagendatei):
    try:
        f=codecs.open(get_datenpfad()+vorlagendatei,mode="r",encoding='utf-8')
    except:
        return '<h1>Konnte die HTML-Vorlage nicht oeffnen!</h1>',"","",""
    kopf=""
    seite=""
    fuss=""
    seitenseparator='<H6 STYLE="page-break-before: always"> </H6>\n' # Hack, der in Firefox 20 und Libreoffice 4.0 funktioniert
    state=0 # suche <body...
    for zeile in f:
        suchzeile=zeile.upper()
        if state==0:
            kopf+=zeile
            if '<BODY' in suchzeile:
                state=1
        elif state==1:
            if '</BODY' in suchzeile:
                state=2
                fuss+=zeile
            else:
                seite+=zeile
        elif state==2:
            fuss+=zeile
    f.close()
    return kopf,seite,fuss,seitenseparator
    
def get_nameselect(tagname='loginklarname'):
    rowdict_list=a_db.get_rowdict_list('SELECT klarname FROM users ORDER by sortiername')
    select='<select name="'+tagname+'">'
    select+='\t<option>-</option>'
    for row in rowdict_list:
        select+='\t<option>'+row["klarname"]+'</option>'
    select+='</select>'
    return select

def get_userklarname(user_nr):
    if type(user_nr) is int and user_nr>0:
        rowdict=a_db.get_one_rowdict('SELECT klarname FROM users WHERE user_nr=?',(user_nr,))
        if rowdict:
            return rowdict["klarname"]
    return ''

def get_user_nr(klarname):
    if klarname:
        rowdict=a_db.get_one_rowdict('SELECT user_nr FROM users WHERE klarname=?',(klarname,))
        if rowdict:
            return rowdict["user_nr"]
        rowdict=a_db.get_one_rowdict('SELECT user_nr FROM useralias WHERE aliasname=?',(klarname,))
        if rowdict:
            return rowdict["user_nr"]
    return 0

def aliasliste(user_nr,trenner=',',klammern=False):
    ergebnis=trenner.join(a_db.get_column_list('SELECT aliasname FROM useralias WHERE user_nr=? ORDER BY aliasname',(user_nr,)))
    if ergebnis and klammern:
        return ' ('+ergebnis+')'
    return ergebnis
    
def bereinige_name(name,space_ok=False):
    name=name.strip()
    zuersetzen=["+",";","&",":","%","<",">","__","__"] if space_ok else [" ","+",";","&",":","%","<",">","__","__"]
    for illegal in zuersetzen:
        name=name.replace(illegal,"_")
    return name



def remove_config(parameter):
    a_db.execute('DELETE FROM einstellungspaerchen WHERE parameter=?',(parameter,))

def headhelper(zeichen,zeichenfarbe="red",titel="",titelfarbe="",hilfetext=""):
    stil=''
    if not zeichen:
        zeichen='&nbsp;'*12
        stil='font-size:125%;'
    else:
        zeichen='<span style="color:'+zeichenfarbe+';font-size:180%">'+zeichen+'</span> '
    if not titel:
        return zeichen
    stil+='color:'+titelfarbe if titelfarbe else ''
    ph='' if not hilfetext else popuphilfe(hilfetext)
    if stil:
        return '<b>'+zeichen+'<span style="'+stil+'">'+titel+'</span></b>'+ph
    return '<b>'+zeichen+titel+'</b>' + ph
    
def popuphilfe(hilfetext,breit=False,final='<br>'):
    klasse=' class="breit" ' if breit else ''
    return '&nbsp;<popuphelp'+klasse+'><span>'+hilfetext+'</span>&nbsp;<b>&#8239;?&#8239;</b>&nbsp;</popuphelp>'+final
    
def text_lur(text1,text2='',text3='',zeilennummer=0):
    '''Erzeugt eine html-zeile, 
        die den ersten Text linksbündig, 
        den zweiten rechtbündig und danach 
        den dritten in Fettschrift zurück gibt'''
    zeilenfarbe='style="background-color:#f8f8f8;"' if zeilennummer%2 else ''
    a='<div class="lurzeile" '+zeilenfarbe+'> <div class="textlinks">'      
    b='</div><div class="textrechts">&nbsp;'
    c='</div><div style="clear:right;clear:left;"></div> </div>\n'
    if text3:
        return a+text1+b+text2+' <b>'+text3+'</b>'+c
    else:
        return a+text1+b+text2+c
        
def heute_plus_delta_datum(delta):
    return timestamp2isodatum(ts_verschiebe(timestamp,delta))
    
def timestamp2isodatum(ts):
    return time.strftime('%Y-%m-%d',ts)

def ts_verschiebe(ts,delta_tage):
    return time.localtime(time.mktime((ts.tm_year,ts.tm_mon,ts.tm_mday+delta_tage,0,0,0,0,0,-1)))
    
def ts_erstellen(jahr,monat,tag):
    return time.localtime(time.mktime((jahr,monat,tag,0,0,0,0,0,-1)))
    
def importlog(txt,comment="",err=""):
    if comment:
        txt+=": "+comment
    if txt=="":
        txt=err
    if txt: logliste.append(str(txt))
    if err: errliste.append(str(err))

def importlogfinish():
    f=codecs.open("data/import.log",mode="w",encoding='utf-8')
    for row in logliste:
        f.write(row+'\n')
    f.close()
    f=codecs.open("data/import.err",mode="w",encoding='utf-8')
    for row in errliste:
        f.write(row+'\n')
    f.close()
            
def manage_gruppenrechte():
    for level in range(21):
        for gruppe in ["seki","kl"]:
            set_gruppenrechte(gruppe,level,1 if level<=LEVEL_ATTESTOK else 0) # willkürliche Standardwerte

def get_gruppenrechte(gruppe,level):
    return 1 if a_db.get_first_element('SELECT flag FROM gruppenrechte WHERE gruppe=? AND level=?',(gruppe,level),0) else 0
    
def set_gruppenrechte(gruppe,level,flag):
    a_db.execute('INSERT OR REPLACE INTO gruppenrechte (gruppe,level,flag) VALUES (?,?,?)',(gruppe,level,flag))
        
def get_importerrors():
    return "  "+"\n  ".join(errliste)

def name_mit_href(name,pid=0,popup=""):
    '''liefert einen String, der einen <a link> passend zum Namen enthält</a>'''
    if len(popup)>0:
        klasse=' class="tooltip" ' 
        popup='<span>'+popup+'</span>'
    else:
        klasse=''
    zielkurs=alphabetisch[0]
    for kandidat in alphabetisch[1:]:
        if name[0] in kandidat:
            zielkurs=kandidat
            
    linkziel='"a_main.py?highlight='+str(pid)+'&kurs='+zielkurs+'"'
    return '<a '+klasse+' href='+linkziel+'>'+name+popup+'</a>'
        
def get_datenpfad():
    return __datenpfad__    
    
def get_logo():
    import a_session as a_s
    # nur für die GUI-Infos - nicht für CLI-Anwendungen
    popuptitel= ' title="'+PROGRAMMVERSION+'\nDB-Pfad:'+get_datenpfad()+'\nKleinste Klasse:'+str(a_db.get_config_int('minimum-js'))+ \
                '\nIhre Rechte:'+str(a_s.aS.rechte)+' maxaktion:'+str(a_s.aS.maxaktion)+' user_nr:'+str(a_s.aS.user_nr)+ \
                '\nIhre ip:'+get_client_ip() + \
                '\nSchuljahre: aktuell='+str(reales_schuljahr())+'\n anzuzeigen='+str(a_s.aS.anzeigeschuljahr)+ \
                '\n konfiguriert:'+a_db.get_config('intern_gueltiges_schuljahr','nix')+ \
                '\n Flags: real='+str(a_s.aS.zeige_reales_schuljahr)+',konf='+str(a_s.aS.zeige_konfiguriertes_schuljahr)+ \
                  '" '
    logo='<img width="32" height="32" alt="A!" '+popuptitel+' src="/ab.img/ab_icon.png" />'
    return logo
    # '\nHalbjahr: '+str(a_s.aS.anzeigehalbjahr)+' differenziere HJe: '+str(a_s.aS.differenziere_halbjahre) +
    
def starttiming(name):
    global timing_dict
    prt('\n<!-- ZEITMESSUNG: Start von '+name+' -->')
    timing_dict[name]=time.time()
    
def endtiming(name):
    dauer=time.time()-timing_dict[name]
    prt('<!-- ZEITMESSUNG: Dauer von '+name+': '+str(int(dauer*1000))+'ms -->')
    return dauer
    
def htmlstandard_einleitung():
    return('''Content-Type: text/html\n\n<!DOCTYPE html><html> \n''')
def htmlstandard_headstart():
    favi='<link rel="icon" href="/favicon2.ico" />' if ".dev" in basepath else ''
    return('''<head>
    <link rel="stylesheet" href="/a.css" type="text/css" />
    <meta http-equiv="content-type" content="text/html; charset=utf-8" />\n
    <meta http-equiv="cache-control" content="no-cache">'''+favi)
    
##################  
## Startaktionen##
##################
# 1. db suchen
basepath = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))+'/'
allepfade=[basepath+'cgi-bin/data/','data/','cgi-bin/data/','cgi-bin/','./'] # alle Pfade ab html-root und cgi-bin, jeweil mit und ohne data-Unterverzeichnis
for datenpfad in allepfade:
    if os.path.isfile(datenpfad+'absent.sq3'):
        break
else:
    import a_db_anlegen
    a_db_anlegen.db_create()
    a_db_anlegen.lege_admin_an()
    prt(htmlstandard_einleitung()+htmlstandard_headstart()+'''
    </head><body><h1>Datenbank wurde neu angelegt!</h1>
    Grund: Es wurde keine bestehende Datenbank gefunden.<br>Der Administrator wurde mit dem Passwort pass8 angelegt!<br>
    '''+a_db_anlegen.printresult+'<br><br>Zum Fortfahren diese Seite neu <a href="/index.html">laden</a>!</body>')
    sys.exit()
# 2. db verbinden   
db_conn=a_db.db_verbinden(datenpfad+'absent.sq3')
db_cursor=db_conn.cursor()
__datenpfad__=datenpfad

# 3. db-schema ggf. aktualisieren
db_version=a_db.get_config('intern_db_version')
if db_version<'2014-11-09':         # dieses Datum muss in a_db_anlegen auch definiert sein!
    import a_db_anlegen             # ist ja nur einmal nach jedem DB-Upgrade notwendig
    a_db_anlegen.upgradedb(db_conn)

alpha_einlesen()

if a_db.get_config_int("minimum-js")<0: # nicht gesetzt: -1
    a_db.set_config('minimum-js','10')  # Standardeinstellung ab Apr. 2014 um problemlos alle Schulen einzuladen :-)
    
users=0
try:
    users=a_db.get_first_element('SELECT count(kuerzel) FROM users',default=0)
except:
    prt(htmlstandard_einleitung()+htmlstandard_headstart()+'''
    </head><body><h1>Datenbank nicht initialisiert</h1></body>''')
    sys.exit()

if users<1:
    prt(htmlstandard_einleitung()+htmlstandard_headstart()+'''
    </head><body><h1>Datenbank enthält keine User</h1></body>''')
    sys.exit()

now=time.time() #sec since epoche
timestamp=time.localtime(now)
heutedatum=timestamp2isodatum(timestamp)    # im Format 2014-04-21
startjahr=timestamp.tm_year # Jahr des Schuljahresbeginns
if timestamp.tm_mon<9: startjahr-=1

logliste=[]
errliste=[]
timing_dict={}

if not a_db.get_config('intern_gueltiges_schuljahr',''):
    a_db.set_config('intern_gueltiges_schuljahr',reales_schuljahr() )

__cgi__=cgi.FieldStorage()
        
if __name__ == '__main__':
    prt(htmlstandard_einleitung()+htmlstandard_headstart()+'''
    <meta http-equiv="refresh" content="610; URL=a_main.py" />
    </head><body><h1>Unzulässiger Aufruf!</h1>Selbsttest: ''')
    prt(heutedatum+' ')
    prt(str(users)+' User in der DB ')
    # Ein Schreibtest
    a_db.set_config("intern_Selbsttest",str(timestamp))
    # Wenn wir hier ohne Exception ankommen läuft der db-Zugriff
    prt('+ Schreibzugriff klappt!')
    prt('</body>')

